/* $Id: beacon.h 442 2011-02-20 09:57:44Z oh2gve $
 * 
 * Copyright 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef BEACON_H
#define BEACON_H


// Own stuff.
#include "cfg.h"
#include "dupefilter.h"
#include "connection.h"
#include "aprspacket.h"
#include "packet.h"

// std stuff.
#include <vector>
#include <string>
#include <list>
#include <ctime>
#include <map>

// wx stuff.
#include <wx/thread.h>
#include <wx/string.h>



/// Beacon packet handler.
class Beacon : public wxThread
{
public:
		
	/// Constructor.
	Beacon(Cfg::Configuration const& cfg, DupeFilter* dupefilter);
	
	/// Destructor.
	~Beacon();
	
	/// Main function of the thread.
	virtual ExitCode Entry();
	
	/// Call this after Run().
	void waitForInit();
	
	/// Call this after waitForInit returns.
	bool operator!() const;
	
	/// Terminates the thread.
	void terminate();
	
	/// Add connection to list of connections where beacons are sent.
	/**
	 * \param con Connection to add.
	 * \param conid Id char of the connection.
	 * \param path Path to be used with the connection.
	*/
	bool addConnection(Connection* con, char const& condif, std::vector<wxString> const& path);
	
private:

	/// Reads object file contents into objects_.
	bool readObjectFile();
	
	/// Reads status flag.
	bool keepGoing();
	
	/// Private data access serializer.
	wxMutex mutex_;
	
	/// Standard initialization flag.
	bool initialized_;

	/// Status flag.
	bool keepgoing_;
	
	
	/// Duplicate contoller.
	DupeFilter* dupefilter_;
	
	/// Body of all beacon packets.
	std::string beaconBody_;
	
	/// Callsign of this IGate.
	std::string const owncall_;
	
	/// Beacon interval of own location packet in seconds.
	time_t const myInterval_;
	
	/// Name of object file.
	std::string const objFileName_;
	
	/// Data related to one object. Times are in seconds.
	struct ObjectPacket
	{
		AprsPacket* packet;
		time_t offset;
		time_t interval;
	};
	
	/// Storage of object packets.
	std::list<ObjectPacket> objects_;
	
	/// Pair of connection and packet that goes into it.
	struct BeaconPacket
	{
		// Packet to beacon.
		Packet const* packet;
		
		// Connection for the packet.
		Connection* con;

		/// Beaconing interval in seconds.
		time_t interval;
	};
	
	/// Beaconing list containing all the beaconing we must do.
	std::multimap<time_t, BeaconPacket> beacons_;
};

#endif // BEACON_H
