/* $Id: filter.h 433 2011-02-14 16:27:15Z oh2gve $
 * 
 * Copyright 2009, 2010, 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef FILTER_H
#define FILTER_H


// own stuff.
#include "filterelement.h"
#include "aprspacket.h"

// std stuff.
#include <vector>
#include <string>
#include <map>
#include <ctime>



/// Packet filter for one connection.
/**
 * Handles static filter given in config file and dynamic filter based on
 * packets heard from the connection.
 */
class Filter
{
public:
		
	/// Constructor.
	/**
	 * Eats static filter elements for breakfast.
	*/
	Filter(std::vector<FilterElement> const& elements);
	
	/// Destructor.
	~Filter();
	
	/// Adjusts dynamic filter based on given incoming packet.
	void incomingPacket(AprsPacket const* packet);
	
	/// Checks if packet passes the filter.
	bool outgoingPacket(AprsPacket const* packet);
	
private:
	
	/// Check given packet against our static filter.
	bool passesStaticFilter(AprsPacket const* packet);
	
	/// Check given packet against our dynamic filter.
	bool passesDynamicFilter(AprsPacket const* packet);
	
	/// Add target to client list.
	/**
	 * If it is already on the list, updates timestamp.
	*/
	void updateClientList(std::string const& call);
	
	/// Checks if target is on client list.
	bool onClientList(std::string const& call) const;
	
	/// Flush old clients from list.
	void flush();


	/// Elements of static filter.
	std::vector<FilterElement> const elements_;

	/// Client list of our connection.
	std::map<std::string, time_t> clients_;
};

#endif // FILTER_H
