/* $Id: hold.h 427 2011-01-23 22:19:13Z oh2gve $
 * 
 * Copyright 2011 Tapio Aaltonen
 * 
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/

#ifndef HOLD_H
#define HOLD_H


// own stuff.
#include "packet.h"
#include "aprspacket.h"

// std stuff.
#include <ctime>
#include <list>



/// Hold for outgoing packets.
/**
 * Use this to hold packets for a while before they go out to RF.
*/
class Hold
{
public:
	
	/// Constructor.
	Hold(time_t const& holdtime);
	
	/// Destructor.
	~Hold();
	
	/// Put packet in hold.
	void put(Packet* packet);
	
	/// Get packet from hold.
	/**
	 * \return NULL if no packet can be returned.
	*/
	Packet* get();
	
	/// Inform hold that copies of this packet should be dropped.
	void inform(AprsPacket* packet);
	
private:
	
	/// Amount of seconds to keep packet in hold.
	time_t const holdtime_;
	
	/// Queue of outgoing packets.
	std::list<Packet*> hold_;
};
	
#endif // HOLD_H
