/* $Id: rfcon.h 445 2011-02-21 09:17:48Z oh2gve $
 *
 * Copyright 2007 Antti Seppänen
 * Copyright 2007, 2008, 2009, 2010, 2011 Tapio Aaltonen
 *
 * 
 * This file is part of aprsg.
 *
 * aprsg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * aprsg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with aprsg.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * APRS is a registered trademark of APRS Software and Bob Bruninga, WB4APR.
*/


#ifndef RFCON_H
#define RFCON_H


// own stuff.
#include "connection.h"
#include "packetlog.h"
#include "dupefilter.h"
#include "filter.h"
#include "hold.h"

// std stuff.
#include <string>
#include <vector>



/// Common interface for RF connections.
class RFCon : public Connection
{
public:
	
	/// Constructor.
	RFCon(Cfg::ConnectionCfg const& cfg, char const& conId, std::string const& owncall,
	      Packetlog* packetlog, DupeFilter* dupefilter);

	/// Destructor.
	virtual ~RFCon();

	/// This is the entry point of thread. "Main()"-function.
	/**
	 * Must be implemented in child class.
	 */
	virtual ExitCode Entry() = 0;

protected:

	/// Put data received from connection to input queue.
	/**
	 * Base class inputPush + dynamic filter and hold handling.
	 */
	void input(AprsPacket* const& data);

	/// Check output queue for data to be transmitted.
	/**
	 * Base class outputPop + filtering and hold time handling.
	 */
	virtual AprsPacket* output();

private:
	
	/// My callsign.
	std::string owncall_;
	
	/// Connections output path.
	std::vector<std::string> mypath_;
	
	/// Packet log service.
	Packetlog* packetlog_;
	
	/// Duplicate filter service.
	DupeFilter* dupefilter_;
	
	/// Filter of this connection.
	Filter filter_;
	
	/// Hold for outgoing packets.
	Hold hold_;
};

#endif // RFCON_H
